/*****************************************************************************

 JOCO's ZX spectrum emulator.
 tapes.c: Reading ZX spectrum TAP files. 
 Only the loading funtcions are implemented.
 SAVE is not possible yet.

 see the file read.me for details/legal

******************************************************************************/
#include <stdio.h>

#include "z80cpu.h"
#include "zx_hw.h"

/* #define DEBUG */

char 	TapeName[256]="";
int	Opened=0;
int	TapeSize=0;
FILE	*Tape;
int	ERROR;
byte	*TBF=NULL;
#ifdef DEBUG
FILE	*dbg;
#endif

/* a simple filesize function  */
long filesize(FILE *stream)
{
   long curpos, length;

   curpos = ftell(stream);
   fseek(stream, 0L, SEEK_END);
   length = ftell(stream);
   fseek(stream, curpos, SEEK_SET);
   return length;
}

/* generally used variables */
word	tmp,CurrLen,RealLen;
dword	CurrPos;

/* close the tapefile */
void CloseTape() {
 if (!Opened) return;
 fclose(Tape);
 Opened=0;	/* closed ! */

#ifdef DEBUG
 fclose(dbg);
#endif
}

/* open a spetrum tapefile (.TAP) */
int OpenTape(const char *name) {
 if (Opened) CloseTape();
#ifdef DEBUG
  dbg=fopen("debug.txt","w");
#endif

 Opened=0;
 if (Tape=fopen(name,"rb")) {
   if (!(TapeSize=filesize(Tape))) {
    fclose(Tape);
    return 0;
   }
   CurrPos=0;	/* current position */
   Opened=1; 	/* it's opened      */
   return 1;
 }
 return 0;
}

/* alloc some mem for the next spectrum file. 64K must be enough in all
   the cases */
int AllocTapeBuff() {
 if (TBF) return 0;
 TBF=(byte *)malloc(65536);
 if (!TBF) return 0;
 return 1; 
}

/* free the allocated tapefile buffer */
void FreeTapeBuff() {
 if (!TBF) return;
 free(TBF);
}

/* seek to the next spectrum chunk in the tapefile */
void SeekToNext() {
  ERROR=1;
  if (!Opened) return;
  /* move back to the start position if we reached the end of the file */
  if ((CurrPos==TapeSize)||(feof(Tape))) fseek(Tape,0,SEEK_SET);
  /* get the next lenght of chunk */
  fread(&CurrLen,1,2,Tape);
  /* oops! something wrong. Spectrum does not have long files.... */
  if (CurrLen>65535) return;
  /* read the chunk into the buffer */
  fread(TBF,1,CurrLen,Tape);
#ifdef DEBUG
  fprintf(dbg,"SeekToNext, currlen=%d\n",CurrLen);
#endif
}

/* load the next file. This function must work with the spectrum emulator
together, cause there is a patch in the specrum ROM routine. So the 
LoadNext is called when we need to read a file form the tape */
void LoadNext() {
  if (!Opened) return;

  Z_IX&=0xffff;	/* regiter inits */
  Z_DE&=0xffff;
  SeekToNext(); /* seek to the next spectrum file */

#ifdef DEBUG
  fprintf(dbg,"Load AF=%04x IX=%04x DE=%04x\n",Z_AF,Z_IX,Z_DE);
#endif

  /* wrong filetype, skip it */
  if ((Z_AF>>8)!=TBF[0]) {
	Z_AF&=0xffbe; // Cy=0, Z=0
	return;
  }

  /* length of the file */
  RealLen=Z_DE;
  /* a safety correction: if something is wrong in the tapefile,
     you'll get a TAPE ERROR in the emulator, but nothing else
     happens */
  if (Z_DE>CurrLen-2) RealLen-=2;
  /* load .... */
  memcpy(&(Z80mem[Z_IX]),&(TBF[1]),RealLen);
  // Refresh Screen, sometimes it's needed 
  RefreshScreen();
  tmp=Z_DE;
  /* update the registers ... done */
  (word)Z_IX+=RealLen;
  (word)Z_DE-=RealLen;
  Z_AF|=1; // Cy=1

#ifdef DEBUG
  fprintf(dbg,"LEnd AF=%04x IX=%04x DE=%04x\n",Z_AF,Z_IX,Z_DE);
#endif
}

/* a cover for saving the regs and prepare for the load */
void LoadFile() {
 __asm __volatile__("
	xchgb	%al,%ah
	movw	%ax,_Z_AF
	movw	%dx,_Z_DE
	pushal
	call	_LoadNext
	popal
	movw	_Z_AF,%ax
	movw	_Z_DE,%dx
	xchgb	%al,%ah	
 ");
}

